#!/usr/local/bin/perl 
#
#               -------------
#		   Links
#               -------------
#               Links Manager
#
#         File: build.pl
#  Description: This routine validates the links. It checks to see that a return code of 
#               200 is returned for each link and flags any errors. Return codes of 404 (File
#               Not Found) should be updated. Return codes of 500, should be checked but might still 
#               be valid. It will store the results in a log file specified in the config file.
#
#               NOTE: This routine requires the LWP module and NET::FTP available from 
#					   CPAN: http://perl.com/CPAN-local/
#
#               It is also recommend that you run this from either a shell, or a non-parsed
#               header script, as it can take "VERY" long to complete. With 150 links, I averaged
#               10-15 mins on a 28.8 connection. 
#       Author: Alex Krohn
#        Email: alex@gossamer-threads.com
#          Web: http://www.gossamer-threads.com/
#       :
#     Ƨ@:konica  Hc : tengcy@pcmail.com.tw
#      Version: 4.0a
#
# COPYRIGHT NOTICE:
#
# Copyright 1997 Gossamer Threads Inc.  All Rights Reserved.
#
# This program is being distributed as shareware.  It may be used and
# modified free of charge for personal, academic, government or non-profit
# use, so long as this copyright notice and the header above remain intact.
# Any commercial use should be registered.  Please also send me an email,
# and let me know where you are using this script. By using this program 
# you agree to indemnify Gossamer Threads Inc. from any liability.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  Obtain written permission before redistributing this
# program over the Internet or in any other medium.  In all cases
# copyright and header must remain intact.
#
# Please check the README file for full details on registration.
# =====================================================================

# Required Librariers
# --------------------------------------------------------
eval {
	($0 =~ m,(.*)/[^/]+,)   && unshift (@INC, "$1");	# Get the script location: UNIX /
	($0 =~ m,(.*)\\[^\\]+,) && unshift (@INC, "$1");	# Get the script location: Windows \

    use LWP::UserAgent;
    require "$db_lib_path/links.cfg";				# Change this to full path to links.cfg if you have problems.
    require "$db_lib_path/db.pl";
};
if ($@) {
    print "Content-type: text/plain\n\n";
    print "Error including libraries: $@\n";
    print "Make sure they exist, permissions are set properly, and paths are set correctly.";
	exit;
}

# ========================================================

eval { &main; };					        # Trap any fatal errors so the program hopefully 
if ($@) { &cgierr("fatal error: $@"); }     # never produces that nasty 500 server error page.
exit;   # There are only two exit calls in the script, here and in in &cgierr. 

sub main {
# -----------------------------------------------------
    $| = 1;     # Must flush output.

	$use_html = 0;
	if ($ENV{'REQUEST_METHOD'}) { $use_html = 1; }

    open (DB, "<$db_file_name") or &cgierr("~b verify_links. Lk} db ɮ: $db_file_name. z: $!");
        @lines = <DB>;
    close DB;

	my ($time) = &get_time;
	my ($date) = &get_date;

# Print full headers for nph scripts.
	$nph = 1;
	&html_print_headers  if ($use_html);
	
	$use_html ?
    	print qq|
<html>
<head>
<title>s</title>
</head>
<body bgcolor=#ffffff>
<p><font face=Verdana size=4><strong>s . . .</strong></font>
<p>s}lb: $time
<p>ݭnˬd |, $#lines+1, qq| s. oݭnܤ[..еy:
<p><ul>| :
		print "sMw: $time\n ", $#lines+1, " s. еy: \n";

	local (@data, $children);

    open (LOG, ">>$db_verify_path");
	print LOG "}l ", $#lines+1, " sb $time, $date.\n";
    LINE: foreach $line (@lines) {
        if ($line =~ /^$/) { next LINE; }
        if ($line =~ /^#/) { next LINE; }
        chomp ($line);          

        (@data) = &split_decode($line);

		if ($data[$db_url] !~ /^http/) { next LINE; }
		
		if ($db_use_fork) {
			if ($children > $db_max_children) {
				wait; $children--;
			}
			&spawn ($data[0], $data[$db_url]);
		}
		else {
			&check_link ($data[0], $data[$db_url]);
		}
    }

	$time = &get_time; $date = &get_date;
	print LOG "b $time, $date";
    close LOG;
	$use_html ?
		print "</ul><p>b $time.</body></html>" :
		print "\nb $time.\n";
}

sub spawn {
# -----------------------------------------------------
	my ($key, $url) = @_;
	my ($pid);

	if (!defined($pid = fork)) {
		&cgierr ("cannot fork: $!");
	}
	elsif ($pid) {
		$children++;
		return;
	}
	else {
		&check_link ($key, $url);
		exit;
	}
}

sub check_link {
# -----------------------------------------------------
	my ($key, $url) = @_;

    my ($ua) = new LWP::UserAgent;
    $ua->agent("Links Manager 1.1 : http://www.gossamer-threads.com/scripts/links/"); # Please do not change.
	
	my ($req) = new HTTP::Request ('HEAD', $url);
   	my ($res) = $ua->request($req);
	$code = $res->code;

	$use_html ?
		print qq|<li>: <a href="$url" target="_blank">$key</a> . . .| :
		print qq|: $key . . .|;		
	print LOG qq|: $key . . . |;

	if ($res->is_success) {
		$use_html ?
			print qq|\ (<a href="codes.htm#$code" target="_blank">$code</a>).\n| :
			print qq|\ ($code).\n|;
		print LOG "\ ($code).\n";
	}
	else {
		$use_html ?
			print qq| (<a href="codes.htm#$code" target="_blank">$code</a>). 
				<a href="$db_script_url?modify_form_record=1&modify=$key">ק</a>
				<a href="$db_script_url?delete_records=1&$key=delete">R</a>.\n|  :
			print   qq| ($code).\n|;			
		print LOG " ($code).\n";
	}
}